LINUX = 'Linux'
OSX = 'Darwin'
WINDOWS = 'Windows'

SYS_PATH = '/'
PUBLIC_DNS = '8.8.8.8'
LOCALHOST = '127.0.0.1'

CPU_MODEL_LINUX_CMD = "cat /proc/cpuinfo | grep 'model name' | sort -u | cut -d: -f2"
CPU_MODEL_OSX_CMD = "sysctl -n machdep.cpu.brand_string"



__all__ = ('is_iterable', 'is_sequence', 'is_mapping', 'is_callable')

from typing import Iterable, Sequence, Mapping


def is_iterable(o):
    """Check if the given object ``o`` is of type Iterable
    :param o:
    :return:
    """
    return isinstance(o, Iterable)


def is_sequence(o):
    """Check if the given object ``o`` is of type Sequence

    :param o:
    :return:
    """
    return isinstance(o, Sequence)


def is_mapping(o):
    """Check if the given object ``o`` is of type Mapping

    :param o:
    :return:
    """
    return isinstance(o, Mapping)


def is_callable(o):
    """Check if the given object ``o`` is of type callable

    :param o:
    :return:
    """
    return callable(o)



from unittest import mock

import pytest

from pyrish.shell import run_shell
from pyrish.shell.exceptions import (
    CommandNotFoundError,
    UnexpectedShellError,
    MissingPermissionError
)


@mock.patch('pyrish.shell.subprocess')
def test_run_shell_succeed(subprocess):
    subprocess.getstatusoutput.return_value = (0, 'random result')
    output = run_shell('ls -a')
    assert output == 'random result'


@mock.patch('pyrish.shell.subprocess')
def test_run_shell_fails_silently(subprocess):
    subprocess.getstatusoutput.return_value = (1, 'command not found')
    output = run_shell('ipconf', silent=True)
    assert output is None


@pytest.mark.parametrize('cmd, code, exception', [
    ('ipconf', 127, CommandNotFoundError),
    ('cat /bin/junk', 1, UnexpectedShellError),
    ('grep -x lk', 2, UnexpectedShellError),
    ('/etc/hosts', 126, MissingPermissionError),
    ('/bin/kill $$', -15, UnexpectedShellError),
])
@mock.patch('pyrish.shell.subprocess')
def test_run_shell_fails_with_exception(subprocess, cmd, code, exception):
    subprocess.getstatusoutput.return_value = (code, 'random error message')
    with pytest.raises(exception) as e:
        run_shell(cmd)
        assert e.error == 'random error message'
        assert e.code == code
        assert e.cmd == cmd



import pytest

from pyrish.types import is_iterable, is_sequence, is_mapping, is_callable


@pytest.mark.parametrize('obj, result', [
    ([1, 2, 3, 4], True),
    ([], True),
    ((9, 7), True),
    (iter([1, 2, 3, 4]), True),
    (range(45, 50), True),
    ('sfgdasdg', True),
    ({3, 4}, True),
    ({'a': 1}, True),
])
def test_is_iterable(obj, result):
    assert is_iterable(obj) == result


@pytest.mark.parametrize('obj, result', [
    ([1, 2, 3, 4], True),
    ([], True),
    ((9, 7), True),
    (iter([1, 2, 3, 4]), False),
    (range(45, 50), True),
    ('sfgdasdg', True),
    ({3, 4}, False),
    ({'a': 1}, False),
])
def test_is_sequence(obj, result):
    assert is_sequence(obj) == result


@pytest.mark.parametrize('obj, result', [
    ([1, 2, 3, 4], False),
    ((9, 7), False),
    ({}, True),
    ({'a': 1}, True),
])
def test_is_mapping(obj, result):
    assert is_mapping(obj) == result


@pytest.mark.parametrize('obj, result', [
    ([1, 2, 3, 4], False),
    (Exception, True),
    (lambda x: x + 1, True),
    (map, True)
])
def test_is_callable(obj, result):
    assert is_callable(obj) == result



__all__ = (
    'isplit',
    'first',
    'last',
    'last_n',
    'filter_n'
)

import itertools as it
from typing import (
    Any,
    Callable,
    Generator,
    Iterable,
    Optional,
    Iterator
)

from collections import deque
from re import finditer

from .exceptions import EmptyIterableError
from .types import is_sequence


def isplit(s: str, sep: Optional[str] = '') -> Iterator[Any]:
    """Return a list of the words from ``s``, using ``sep`` as the delimiter.
    If ``sep`` is not specified or is None, whitespace is used a separator
    and empty strings are removed from the result.

    :param (str) s: string to be splited into the list of words
    :param (str) sep: delimiter
    :return: Iterator[Any]

    Usage::

        >>> first(isplit('#ln;dsfg#dfsgs#', sep='#'))
        'ln;dsfg'
        >>> list(isplit('#ln;dsfg#dfsgs#', sep='#'))
        ['ln;dsfg', 'dfsgs']
    """

    def finder_pattern():
        if not sep:
            return r'.?'
        return r'[^{sep}]*'.format(sep=sep)

    pattern = finder_pattern()
    items = (x.group(0) for x in finditer(pattern, s) if x.group(0))
    return items


def first(iterable: Iterable, condition: Optional[Callable] = None) -> Any:
    """Returns the first item in the `iterable` that
    satisfies the `condition`.

    If the condition is not given, returns the first item of
    the iterable.

    Raises `EmptyIterableError` if iterable is empty or
    no item satisfying the condition is found.

    :param iterable:  an iterable object
    :param condition: a test function used to filter out items from the iterable
    :return: Any

    Usage::

        >>> first((1,2,3), condition=lambda x: x % 2 == 0)
        2
        >>> first(range(3, 100))
        3
        >>> first( () )
        Traceback (most recent call last):
        ...
        EmptyIterableError
    """
    try:

        if not condition:
            if is_sequence(iterable):
                return iterable[0]
            return next(iterable)

        return next(x for x in iterable if condition(x))
    except (StopIteration, IndexError) as _:
        raise EmptyIterableError()


def last(iterable: Iterable) -> Any:
    """Returns the last item of the given the ``iterable``.

    :param iterable:  an iterable object
    :param condition: a test function used to filter out items from the iterable
    :return:
    """
    try:
        result = last_n(iterable, 1)
        return next(result)
    except StopIteration as _:
        raise EmptyIterableError()


def last_n(iterable: Iterable, n: int) -> Generator[Any, None, None]:
    """Returns the last ``n`` items of the given  ``iterable``.

    :param iterable:  an iterable object
    :param n: the number of items to be returned
    :return: Generator[Any, None, None]

    Usage::

        >>> data = [1,234,234,235,3465,342,3,4,2345,2345,2345]
        >>> last_n(data, 3)
        <generator object last_n at 0x109653830>
        >>> list(last_n(data, 3))
        [2345,2345,2345]
    """

    if is_sequence(iterable):
        iterable = iterable[-n:]
    else:
        iterable = deque(iterable, maxlen=n)

    for i in iterable:
        yield i


def filter_n(condition: Callable, iterable: Iterable, n: Optional[int] = 0) -> Iterable:
    """Return an iterator yielding the first ``n`` items of iterable that
    satisfies ``condition(item)``. If ``n`` not given or equal zero it returns
    all items that satisfies the test function.

    :param condition: a test function used to filter out items from the iterable
    :param iterable:  an iterable object
    :param n: the number of items to be returned
    :return:

    Usage::

        >>> data = [1,234,234,235,3465,342,3,4,2345,2345,2345]
        >>> filter_n(lambda x: x % 5 == 0, data)
        <filter at 0x102de6630>
        >>> list(filter_n(lambda x: x % 5 == 0, data))
        [235, 3465, 2345, 2345, 2345]
        >>> list(filter_n(lambda x: x % 5 == 0, data, n=3))
        [235, 3465, 2345]
        >>> list(filter_n(lambda x: x % 5 == 0, [], n=3))
        []
    """

    if not n:
        return filter(condition, iterable)

    counter = it.count()

    def _condition_with_counter(x):
        if not condition(x):
            # don't increment counter
            return False
        count = next(counter)
        return count < n

    return filter(_condition_with_counter, iterable)



from typing import Generator

import pytest

from pyrish.exceptions import EmptyIterableError
from pyrish.itertools import isplit, first, last_n, last, filter_n


# @@@@@@ testing isplit @@@@@@

@pytest.mark.parametrize('string, sep, length, expected_list', [
    ('#ln;dsfg####dfsgs##', '#', 2, ['ln;dsfg', 'dfsgs']),
    ('http://www.site.de/verzeichnis/unterverzeichnis', '/', 4,
     ['http:', 'www.site.de', 'verzeichnis', 'unterverzeichnis']),
    ('', '*', 0, []),
])
def test_isplit_non_empty_separator(string, sep, length, expected_list):
    result = isplit(string, sep=sep)
    assert isinstance(result, Generator)
    result_list = list(result)
    assert len(result_list) == length
    assert result_list == expected_list


@pytest.mark.parametrize('string, length, expected_list', [
    ('test', 4, ['t', 'e', 's', 't']),
    ('', 0, []),
])
def test_isplit_empty_separator(string, length, expected_list):
    result = isplit(string)
    assert isinstance(result, Generator)
    result_list = list(result)
    assert len(result_list) == length
    assert result_list == expected_list


# @@@@@@ testing first @@@@@@

@pytest.mark.parametrize('obj, result, condition', [
    (range(3, 100), 3, None),
    (iter((11, 21, 31)), 11, None),
    ([11, 13, 25, 34], 25, lambda x: x % 5 == 0),
])
def test_first(obj, result, condition):
    x = first(obj, condition)
    assert x == result


@pytest.mark.parametrize('obj, condition', [
    ([], None),
    ([11, 13, 27, 31], lambda x: x % 5 == 0),
])
def test_first_raises_stop_iteration(obj, condition):
    with pytest.raises(EmptyIterableError):
        first(obj, condition)


# @@@@@@ testing last_n @@@@@@

@pytest.mark.parametrize('obj, n, expected_list', [
    ([11, 13, 27, 31, 45, 89], 3, [31, 45, 89]),
    (iter(['wdf', 'wer', 'cvb', 'tuy', 'fgh']), 2, ['tuy', 'fgh']),
    ([11, 13, 27, 31, 45, 89], 7, [11, 13, 27, 31, 45, 89]),
    (iter(['wdf', 'wer', 'cvb', 'tuy', 'fgh']), 0, []),
    ([], 1, [])
])
def test_last_n(obj, n, expected_list):
    result = last_n(obj, n)
    assert isinstance(result, Generator)
    result_list = list(result)
    assert result_list == expected_list


# @@@@@@ testing last @@@@@@

def test_last_with_non_empty_list():
    assert last([11, 13, 27, 31, 45, 89]) == 89


def test_last_with_empty_list():
    with pytest.raises(EmptyIterableError):
        last([])


# @@@@@@ testing filter_n @@@@@@

@pytest.mark.parametrize('condition, obj, n, expected_list', [
    (lambda x: x % 5 == 0, [1, 235, 234, 3, 4, 45, 345, 2345], 3, [235, 45, 345]),
    (lambda x: x % 5 == 0, [1, 9, 234, 3, 4, 2341, 23, 2345, 5], 2, [2345, 5]),
    (lambda x: x % 5 == 0, [1, 15, 234, 3, 4, 2341, 23, 2345, 5], 0, [15, 2345, 5]),
    (lambda x: x % 5 == 0, [5], 4, [5]),
    (lambda x: x % 5 == 0, [], 2, []),
])
def test_filter_n(condition, obj, n, expected_list):
    result = filter_n(condition, obj, n)
    assert list(result) == expected_list



"""
This module tries to provide a ready to consume data that identifies the system as
much as possible. It makes this information available via function APIs.
"""

__all__ = (
    'architecture',
    'cpu_info',
    'cpu_model_name',
    'disk_usage',
    'host_info',
    'internal_ip',
    'is_linux',
    'is_osx',
    'is_win',
    'os_info',
    'os_name',
    'os_release',
    'uptime',
    'virtual_memory_usage',
    '_cpu_linux_model',
    '_cpu_osx_model'
)

import platform
import socket
import time
from typing import Dict, Any

import humanfriendly as hf
import psutil

from ._globals import (LINUX, OSX, WINDOWS, SYS_PATH, PUBLIC_DNS,
                       CPU_MODEL_LINUX_CMD, CPU_MODEL_OSX_CMD, LOCALHOST)
from .shell import run_shell


def os_name() -> str:
    """Returns the OS name, e.g. 'Linux', 'Darwin' etc.
    An empty string is returned if the value cannot be determined.
    """
    return platform.system()


def is_linux() -> bool:
    """Checks if it's a Linux environment

    :return: (bool)
    """
    return os_name() == LINUX


def is_osx() -> bool:
    """Checks if it's OSX environment

    :return: (bool)
    """
    return os_name() == OSX


def is_win() -> bool:
    """Checks if it's Windows environment

    :return: (bool)
    """
    return os_name() == WINDOWS


def os_release() -> str:
    """
    Returns the release version of the os. An empty string is returned
    if the value cannot be determined.

    :return: (str)

    Usage::

        >>> os_release()
        '17.3.0'
    """
    return platform.release()


def architecture() -> str:
    """Returns the system's bit architecture

    Usage::

        >>> architecture()
        '64bit'
    """

    arch, _ = platform.architecture()
    return arch


def os_info() -> Dict[str, str]:
    """Collects information about the installed operating system.

    :return: (Dict[str, str])

    Usage::

        >>> os_info()
        {
            'arch': '64bit',
            'name': 'Darwin',
            'release': '17.3.0',
        }
    """

    info = {
        'name': os_name(),
        'release': os_release(),
        'arch': architecture()
    }
    return info


def uptime(human: bool = False) -> Any:
    """Gives the time since then the system has been working and available.
    The time is returned in seconds. If ``human=True`` uptime
    will be returned as a human readable string.

    :param (bool) human: convert time to human readable sting
    :return: (str/float)

    Usage::

        >>> uptime()
        93222.41144919395
        >>> uptime(human=True)
        '1 day, 1 hour and 54 minutes'
    """

    now = time.time()
    uptime = psutil.boot_time()
    uptime_delta = now - uptime
    if human:
        uptime_delta = hf.format_timespan(uptime_delta)
    return uptime_delta


def virtual_memory_usage(human: bool = False) -> Dict[str, Any]:
    """Return statistics about system memory usage

    :param (bool) human: convert memory size to human readable sting
    :return: (Dict[str, Any])

    Usage::

        >>> virtual_memory_usage(human=True)
        {
            'free': '1.03 GB',
            'total': '17.18 GB',
            'usage_percent': 73.5,
            'used': '14.71 GB'
        }
    """

    info = {}
    memory = psutil.virtual_memory()
    info['total'] = memory.total
    info['used'] = memory.used
    info['free'] = memory.free
    if human:
        info = {k: hf.format_size(v) for k, v in info.items()}
    info['usage_percent'] = memory.percent
    return info


def disk_usage(human: bool = False) -> Dict[str, Any]:
    """Return disk usage statistics

    :param (bool) human: convert memory size to human readable sting
    :return: (Dict[str, Any])

    Usage::

        >>> disk_usage(human=True)
        {
            'free': '459.3 GB',
            'total': '499.96 GB',
            'usage_percent': 7.4,
            'used': '36.75 GB'
        }
    """

    info = {}
    memory = psutil.disk_usage(SYS_PATH)
    info['total'] = memory.total
    info['used'] = memory.used
    info['free'] = memory.free
    if human:
        info = {k: hf.format_size(v) for k, v in info.items()}
    info['usage_percent'] = memory.percent
    return info


def cpu_info(include_logical=False) -> Dict[str, Any]:
    """Returns number of CPU physical [logical] cores and model.

    :return: Dict[str, str]

    Usage::

        >>> cpu_info()
        {
            'cores': 4,
            'model_name': 'Intel(R) Core(TM) i7-7700HQ CPU @ 2.80GHz'
        }
    """

    info = {}
    info['cores'] = psutil.cpu_count(logical=include_logical)
    info['model_name'] = cpu_model_name()
    return info


def _cpu_linux_model():
    """Grabs CPU brand description from a Linux environment"""

    model = run_shell(CPU_MODEL_LINUX_CMD)
    return model


def _cpu_osx_model():
    """Grabs CPU brand description from a OSX environment"""

    model = run_shell(CPU_MODEL_OSX_CMD)
    return model


def cpu_model_name() -> str:
    """Returns CPU brand description that gives
    an overview of the CPU family, model.

    :return: (str)

    Usage::

        >>> cpu_model_name()
        'Intel(R) Core(TM) i7-7700HQ CPU @ 2.80GHz'
    """

    if is_linux():
        return _cpu_linux_model()
    if is_osx():
        return _cpu_osx_model()
    return platform.processor()


def internal_ip() -> str:
    """Return the internal ip address of the host.

    Usage::

        >>> internal_ip()
        '192.168.102.55'
    """
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect((PUBLIC_DNS, 80))
        ip = s.getsockname()[0]
        s.close()
    except OSError as _:
        ip = LOCALHOST
    return ip


def host_info(human: bool = False) -> Dict[str, Any]:
    """ Return a collection of information about server and it's current state.

    :param (bool) human: convert all data to human readable sting
    :return:

    Usage::

        >>> host_info()
        {
            'cpu': {
                'cores': 4,
                'model_name': 'Intel(R) Core(TM) i7-7700HQ CPU @ 2.80GHz'
            },
            'disk': {
                'free': 459298390016,
                'total': 499963170816,
                'usage_percent': 7.4,
                'used': 36753997824
            },
            'os': {
                'arch': '64bit',
                'name': 'Darwin',
                'release': '17.3.0',
            },
            'private_ip': '192.168.102.55',
            'uptime': 93222.41144919395,
            'virtual_memory': {
                'free': 1020211200,
                'total': 17179869184,
                'usage_percent': 73.5,
                'used': 14719655936
            }
        }
        >>> host_info(human=True)
        {
            'cpu': {
                'cores': 4,
                'model_name': 'Intel(R) Core(TM) i7-7700HQ CPU @ 2.80GHz'
            },
            'disk': {
                'free': '459.3 GB',
                'total': '499.96 GB',
                'usage_percent': 7.4,
                'used': '36.75 GB'
            },
            'os': {
                'arch': '64bit',
                'name': 'Darwin',
                'release': '17.3.0',
            },
            'private_ip': '192.168.102.55',
            'uptime': '1 day, 1 hour and 54 minutes',
            'virtual_memory': {
                'free': '1.03 GB',
                'total': '17.18 GB',
                'usage_percent': 73.5,
                'used': '14.71 GB'
            }
        }
    """

    info = {}
    info['os'] = os_info()
    info['uptime'] = uptime(human)
    info['virtual_memory'] = virtual_memory_usage(human)
    info['disk'] = disk_usage(human)
    info['cpu'] = cpu_info()
    info['private_ip'] = internal_ip()

    return info



#!/usr/bin/env python

from pip.req import parse_requirements
from setuptools import setup, find_packages


def read(_file):
    with open(_file) as f:
        content = f.read()
    return content


def iter_requirements(_file):
    requirements = parse_requirements(_file, session=False)
    return [str(r.req) for r in requirements]


__version__ = '0.3.1'
__author__ = "Dhia Abbassi"
__email__ = "contact@dhia.io"
__license__ = "MIT license"

readme = read('README.rst')

requirements = iter_requirements('requirements/prod.lock')
setup_requirements = ('pytest-runner',)
test_requirements = iter_requirements('requirements/dev.lock')

classifiers = (
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Natural Language :: English',
    'Programming Language :: Python :: 3.6'
)

setup(
    name='pyrish',
    version=__version__,
    description="A growing collection of code and utility classes.",
    long_description=f"{readme}",
    author=__author__,
    author_email=__email__,
    url='https://github.com/dhiatn/pyrish',
    packages=find_packages(include=['pyrish']),
    include_package_data=True,
    install_requires=requirements,
    license=__license__,
    zip_safe=False,
    keywords='pyrish',
    classifiers=classifiers,
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=setup_requirements,
)
